/*
 * Camera.cpp
 *
 *  Created on: 2010-11-21
 *      Author: krejziwan
 */

#include "Camera.h"
#include<GL/glut.h>

Camera::Camera()
{
  viewAngle = 0.0;
  aspect = 1.0;
  nearDist = 1.0;
  farDist = 2.0;
  eye = Vector(1.0, 1.0, 1.0);
  u = Vector(0.0, 0.0, 1.0);
  v = Vector(0.0, 1.0, 0.0);
  n = Vector(-1.0, 0.0, 0.0);
}

void Camera :: setModelViewMatrix(void)
{ // load modelview matrix with existing camera values
	GLfloat m[16];
	Vector eVec(eye.getX(), eye.getY(), eye.getZ()); // a vector version of eye
	m[0] =  u.getX(); m[4] =  u.getY(); m[8]  =  u.getZ();  m[12] = -eVec*u;
	m[1] =  v.getX(); m[5] =  v.getY(); m[9]  =  v.getZ();  m[13] = -eVec*v;
	m[2] =  n.getX(); m[6] =  n.getY(); m[10] =  n.getZ();  m[14] = -eVec*n;
	m[3] =  0;   m[7] =  0;   m[11] =  0;    m[15] = 1.0;
	glMatrixMode(GL_MODELVIEW);
	glLoadMatrixf(m); // load OpenGL's modelview matrix
}

void Camera:: set(Vector Eye, Vector look, Vector up)
{	// create a modelview matrix and send it to OpenGL
	eye = Eye; // store the given eye position
        n = eye - look;
	u = up%n; // make u = up X n
	n.normalize(); u.normalize(); // make them unit length
	v = n%u;  // make v =  n X u
	setModelViewMatrix(); // tell OpenGL
}

void Camera:: setShape(real vAng, real asp, real nearD, real farD)
{ // define shape of view volume
  viewAngle = vAng;
  aspect = asp;
  nearDist = nearD;
  farDist = farD;
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(viewAngle, aspect, nearDist, farDist);  // tell OpenGL
}

void Camera:: slide(real delU, real delV, real delN)
{ // slide the camera along its own axes
	eye += delU*u + delV*v +delN*n;
	if (eye.getY() <= 4*BALLRADIUS)
		eye.setY(4*BALLRADIUS);
//  eye.x += delU * u.x + delV * v.x + delN * n.x;
//  eye.y += delU * u.y + delV * v.y + delN * n.y;
//  eye.z += delU * u.z + delV * v.z + delN * n.z;
  setModelViewMatrix(); // tell OpenGL
}

void Camera:: roll(real angle)
{ //roll the camera through angle degrees
	real cs = cos(3.14159265 /180 * angle);
	real sn = sin(3.14159265 /180 * angle);
	Vector t(u); //remember old u
	u = cs*u - sn*v;
//	u.set(cs*t.x - sn*v.x, cs*t.y - sn*v.y, cs*t.z - sn*v.z);
//	v.set(sn*t.x + cs*v.x, sn*t.y + cs*v.y, sn*t.z + cs*v.z);
	v = sn*t + cs*v;
	setModelViewMatrix();
}

void Camera:: yaw(real angle)
{ //yaw the camera through angle degrees
	real cs = cos(3.14159265 /180 * angle);
	real sn = sin(3.14159265 /180 * angle);
	Vector t(n); //remember old u
	n = cs*t + sn*u;
	u = cs*u - sn*t;
	//	n.set(cs*t.x + sn*u.x, cs*t.y + sn*u.y, cs*t.z + sn*u.z);
//	u.set(-sn*t.x + cs*u.x, -sn*t.y + cs*u.y, -sn*t.z + cs*u.z);
	setModelViewMatrix();
}

void Camera:: pitch(real angle)
{ //pitch the camera through angle degrees
	real cs = cos(3.14159265 /180 * angle);
	real sn = sin(3.14159265 /180 * angle);
	Vector t(v); //remember old u
	v = cs*t + sn*n;
	n = cs*n - sn*t;
	//	v.set(cs*t.x + sn*n.x, cs*t.y + sn*n.y, cs*t.z + sn*n.z);
//	n.set(-sn*t.x + cs*n.x, -sn*t.y + cs*n.y, -sn*t.z + cs*n.z);
	setModelViewMatrix();
}

void Camera::rotate(real angle) {
	real cs = cos(3.14159265 /180 * angle);
	real sn = sin(3.14159265 /180 * angle);
	real tHit = -eye.getY()/n.getY();
	Vector look = eye + n*tHit;
	Vector v = eye-look;
	Vector v2 = Vector(cs*v.getX()-sn*v.getZ(), v.getY(), v.getX()*sn+v.getZ()*cs);
	eye = look + v2;
	this->set(eye,look,Vector(0.0,1.0,0.0));

}
