#include "Engine.h"
#include "GraphicDefinitions.h"

TextureManager *TextureManager::instance = NULL;
GameState *GameState::instance = NULL;

const char* Engine::WindowSuggestions::windowName = "SNOOK3D";


char avg[100];

inline void displayMainMenu()
{
GLint matrixMode;
GLboolean lightingOn;
sprintf(avg, "Press 'y' to start game");
	  lightingOn= glIsEnabled(GL_LIGHTING);        /* lighting on? */
	 if (lightingOn) glDisable(GL_LIGHTING);

		 glGetIntegerv(GL_MATRIX_MODE, &matrixMode);  /* matrix mode? */

		 glMatrixMode(GL_PROJECTION);
		 glPushMatrix();
		   glLoadIdentity();
		   gluOrtho2D(0.0, 1.0, 0.0, 1.0);
		   glMatrixMode(GL_MODELVIEW);
		   glPushMatrix();
		     glLoadIdentity();
		     glPushAttrib(GL_COLOR_BUFFER_BIT);       /* save current colour */
		       glColor3f(1, 0, 0);
		       glRasterPos3f(0.3, 0.55, 0.0);
		       for(char *ch = avg; *ch; ch++) {
		       		          glutBitmapCharacter(GLUT_BITMAP_9_BY_15,(int)*ch );
		       		       }
		       sprintf(avg, "Press 'p' to pause");
		       glRasterPos3f(0.3, 0.53, 0.0);
		       for(char *ch = avg; *ch; ch++) {
		       		          glutBitmapCharacter(GLUT_BITMAP_9_BY_15,(int)*ch );
		       		       }
		       sprintf(avg, "Press 'h' to hit the ball");
		       glRasterPos3f(0.3, 0.51, 0.0);
		       for(char *ch = avg; *ch; ch++) {
		       		          glutBitmapCharacter(GLUT_BITMAP_9_BY_15,(int)*ch );
		       		       }
		       sprintf(avg, "Press 'ESC' during pause to exit application");
		       glRasterPos3f(0.3, 0.49, 0.0);
		       for(char *ch = avg; *ch; ch++) {
		       		          glutBitmapCharacter(GLUT_BITMAP_9_BY_15,(int)*ch );
		       		       }
		       sprintf(avg, "For detailed key control check documentation");
		       glRasterPos3f(0.3, 0.47, 0.0);
		       for(char *ch = avg; *ch; ch++) {
		       		          glutBitmapCharacter(GLUT_BITMAP_9_BY_15,(int)*ch );
		       		       }
		      glPopAttrib();
		   glPopMatrix();
		   glMatrixMode(GL_PROJECTION);
		 glPopMatrix();
		 glMatrixMode(matrixMode);
		 if (lightingOn) glEnable(GL_LIGHTING);
}


Engine::Engine()
{

	state		= GameState::getInstance();
	textMan 	= TextureManager::getInstance();
	physics		= new Physics();
	scene 		= new Scene();
	camera 		= new Camera();
	logic		= new GameLogic(physics,scene);

	eventHandler = new EventHandler(physics,camera,state);
	state->subscribe(this);
	state->subscribe(eventHandler);
	state->subscribe(physics);
	state->subscribe(logic);
	showFPS = true;
	elapsedTime=0;
	frames=0;

}

void Engine::init()

{
	state->setCurrentState(GameState::INIT);
	textMan->loadTextures();

	glEnable(GL_TEXTURE_2D);
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glEnable(GL_NORMALIZE);
	glColorMaterial(GL_FRONT, GL_DIFFUSE);
	glEnable(GL_COLOR_MATERIAL);//TODO ciechu poczytaj jak sie korzysta z tego trybu i do czego on wlasciwie sluzy
	glEnable(GL_DEPTH_TEST);
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);	// Clear The Background Color To Black
	   glClearDepth(1.0);				// Enables Clearing Of The Depth Buffer
	    glDepthFunc(GL_LESS);			// The Type Of Depth Test To Do
	    glEnable(GL_DEPTH_TEST);			// Enables Depth Testing
	  //  glShadeModel(GL_SMOOTH);			// Enables Smooth Color Shading
	   // glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);
						// Create light components
	GLfloat ambientLight[] = { 1.0f, 1.0f, 1.0f, 1.0f };
	GLfloat diffuseLight[] = { 0.8f, 0.8f, 0.8, 1.0f };
	GLfloat specularLight[] = { 0.5f, 0.5f, 0.5f, 1.0f };


									// Assign created components to GL_LIGHT0
	glLightfv(GL_LIGHT0, GL_AMBIENT, ambientLight);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuseLight);
	glLightfv(GL_LIGHT0, GL_SPECULAR, specularLight);

	// loads OpenGL lists;
	loadGraphicDefinitions();
}

void Engine::processNotification(ISubject *subject)
{

	if (subject->getType()==ISubject::State)
	{
		GameState *st=(GameState *) subject;
		if (st->getCurrentState() == GameState::MAIN_MENU)
		{

		}
		else if (st->getCurrentState() == GameState::NEW_GAME)
		{

		}
	}

}

void Engine::FPS()
{
	showFPS=!showFPS;
}

void Engine::idle()
{
	if (state->getCurrentState()==GameState::ANIMATION )
	{
		physicsTimer.stop();
		physics->physicsUpdate(physicsTimer.getElapsedTimeInMicroSec());
		physicsTimer.start();
		for(int i=0; i<10000; i++){}
	}
	else if (state->getCurrentState() == GameState::PAUSE_ANI || state->getCurrentState()==GameState::HIT_BALL)
	{
		physicsTimer.stop();
		physicsTimer.start();

	}
}

void Engine::display()
{
	glClearColor(0.0,0.0,0.0,0.0);
	glClear(GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT);	// Clear The Screen And The Depth Buffer

	if (state->getCurrentState()==GameState::MAIN_MENU || state->getCurrentState()==GameState::NEW_GAME)
	{
		displayMainMenu();
	}

	else
	{

	if (state->getCurrentState()==GameState::PAUSE_HIT || state->getCurrentState()==GameState::PAUSE_ANI)
		camera->rotate(0.1);

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	Vector lightPosition(0,1.0,0);
    float l[]= { 0, 5.0, 0 ,1};

    glLightfv(GL_LIGHT0, GL_POSITION, l);
    scene->draw();
	scene->drawShadows(lightPosition);

	glFlush();

	}

	if(showFPS)
	{
		renderTimer.stop();
		elapsedTime+=renderTimer.getElapsedTimeInMilliSec();
		if (elapsedTime >=1000)
		{

			frames=0;
			 elapsedTime=0;
		}

	 GLint matrixMode;
	 GLboolean lightingOn;

 	 sprintf(avg, "FPS: %2.2f", frames*1000/elapsedTime);
	  lightingOn= glIsEnabled(GL_LIGHTING);        /* lighting on? */
	 if (lightingOn) glDisable(GL_LIGHTING);

		 glGetIntegerv(GL_MATRIX_MODE, &matrixMode);  /* matrix mode? */

		 glMatrixMode(GL_PROJECTION);
		 glPushMatrix();
		   glLoadIdentity();
		   gluOrtho2D(0.0, 1.0, 0.0, 1.0);
		   glMatrixMode(GL_MODELVIEW);
		   glPushMatrix();
		     glLoadIdentity();
		     glPushAttrib(GL_COLOR_BUFFER_BIT);       /* save current colour */
		       glColor3f(1, 0, 0);
		       glRasterPos3f(0.01, 0.97, 0.0);
		       for(char *ch = avg; *ch; ch++) {
		       		          glutBitmapCharacter(GLUT_BITMAP_8_BY_13,(int)*ch );
		       		       }
		      glPopAttrib();
		   glPopMatrix();
		   glMatrixMode(GL_PROJECTION);
		 glPopMatrix();
		 glMatrixMode(matrixMode);
		 if (lightingOn) glEnable(GL_LIGHTING);


		 frames++;

		 renderTimer.start();
	}

}

void Engine::resize(int Width, int Height)
{
	glViewport(0, 0, Width, Height);		// Reset The Current Viewport And Perspective Transformation
}

void Engine::handleKeyboardEvent(unsigned char key, int x, int y)
{
	eventHandler->handleKeyboardEvent(key,x,y);
}


void Engine::handleMouseEvent(int button, int state,int x, int y)
{
	eventHandler->handleMouseEvent(button, state, x, y);
}

void Engine:: run() {
	  camera->setShape(50,DEF_WIDTH/DEF_HEIGHT,0.1,30);
	  //set Camera eye, look, up
	  camera->set(Vector(0,1.0,-2.0), Vector(0,0,0), Vector(0,1,0));
	  state->setCurrentState(GameState::MAIN_MENU);

}



