#ifndef ENGINE_H
#define ENGINE_H


#include "../UI/EventHandler.h"
#include "Camera.h"
#include "../Physics/Physics.h"
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>
#include "INotifiable.h"
#include "IObservable.h"
#include "GameState.h"
#include "TextureManager.h"
#include "GameLogic.h"
#include <iostream>


const float DEF_WIDTH=800.0;
const float DEF_HEIGHT=600.0;

class Engine: public INotifiable
{
public:
	struct WindowSuggestions
	{
		static const int WIDTH	=800;
		static const int HEIGHT	=600;
		static const int POSX 	= 100;
		static const int POSY 	= 100;
		static const char* windowName;
	};

private:
		GameState *state;
		TextureManager *textMan;
		GameLogic *logic;
		Physics * physics;
		Scene *scene;
		EventHandler *eventHandler;
		Camera *camera;
		Timer physicsTimer;
		Timer renderTimer;

		double elapsedTime;
		int frames;
		bool showFPS;

		//void displayMenu();
		//void displayGame();
		//void displayOptions();
				void FPS();


	public:
	
	Engine();

	void display();
	void idle();
	void init();
	void resize(int Width, int Height);
	void handleKeyboardEvent(unsigned char key, int x, int y);
	void handleMouseEvent(int button, int state,int x, int y);
	void run();
	void processNotification(ISubject *s);

	Physics* getPhysics() { return physics;}
	Camera* getCamera() {return camera;}
	Scene* getScene(){ return scene;}
	GameState* getState(){ return state;}


};

#endif
