/*
 * Contact.cpp
 *
 *  Created on: 2010-11-11
 *      Author: krejziwan
 */
#include <cassert>
#include "Contact.h"
#include "../Common/Vector.h"


void Contact::resolveInterpenetration(real duration) {
	//if there is no penetration ignore this step
	if (penetration<=0) return;
	if(!body[1] || !body[1]->isAwake()) {

		real v = body[0]->getVelocity().scalarProduct(normal);
		real d = penetration/v;
		if(v==0.0) {
			body[0]->updatePosition(normal*penetration);
			return;
		}
		assert(v!=0.0);
		body[0]->updatePosition(body[0]->getVelocity()*d);
		return;
	}
	//project velocities of particles to the contact vector, and change position accordingly to the ratio
	//real a2 = body[0]->getVelocity().squareMagnitude();

	//real v1, sum = v1 = body[0]->getVelocity().project(normal).squareMagnitude();
	//we dont have to check if body[1] is null anymore
	/*	if(!body[1]->isAwake()) {
		real v = body[0]->getVelocity().scalarProduct(normal);
		real d = penetration/v;
		if(v==0.0) {
			body[0]->updatePosition(normal*penetration);
			return;
		}
		assert(v!=0.0);
		body[0]->updatePosition(-body[0]->getVelocity()*d);
		return;
	}*/
	body[0]->updatePosition(penetration/2.0*normal);
	body[1]->updatePosition(-penetration/2.0*normal);
	//czy moze powodowac problemy?? jesli by przesunelo bile na inna bile to co?
}

void Contact::resolveVelocity(real duration) {

	real newRelVel, relVel = newRelVel = calculateRelativeVelocity();
	if(relVel>=0) return;

	newRelVel *= restitution;
	if(!body[1]) {
		newRelVel *= cushionRestitution;
//		std::cout << "przed zderzeniem:rel, delta, v" << relVel << " "<< -normal*(relVel*(1+cushionRestitution)) << body[0]->getVelocity();

		body[0]->updateVelocity(normal*(-relVel*2.0));
		//		body[0]->updateVelocity(-normal*(relVel*(1+cushionRestitution)));
		body[0]->setVelocity(body[0]->getVelocity()*cushionRestitution);
//		std::cout << "pos,v,n:"<< body[0]->getPosition() << body[0]->getVelocity() << normal <<'\n';
		//		std::cout.flush();
		return;
	}
	//	real deltaVel = newRelVel - relVel;
	//ilosc pedu przekazany drugiej bili
	Vector velChange = -normal*relVel;
//	std::cout << body[1]->getVelocity() << body[0]->getVelocity() << " b1 b0 przed zmiana" << '\n';

	body[0]->updateVelocity(velChange);
	if(!body[1]->isAwake())
		body[1]->setAwake(true);
	body[1]->updateVelocity(normal*relVel*restitution);
//	std::cout << body[1]->getVelocity() << body[0]->getVelocity() << body[1]->getNumber() << ' ' <<  body[0]->getNumber() << " predkosci" << '\n';
	//	std::cout.flush();

}

void Contact::resolveGroupVelocity(real duration, Vector normal, RigidBody * b) {
	Contact c = *this;
	RigidBody * left = b->findLeft(-normal);
	RigidBody * right = b->findRight(-normal);
	if(left && right) {

		if(left==right) {
//			std::cout << "Group resolving direct hit: " << left->getNumber() << ' ' << b->getNumber() << '\n';
			c = Contact((b->getPosition()-left->getPosition()).unit(),b,left);
			if(c.calculateRelativeVelocity()<0)
			c.resolve(duration);
			//			std::cout << b->getVelocity();

			return;
		}
//		std::cout << "Group resolving: " << left->getNumber() << ' ' << right->getNumber() << ' ' << b->getNumber() << '\n';
		Vector lNormal = (b->getPosition()-left->getPosition()).unit();
		Vector rNormal = (b->getPosition()-right->getPosition()).unit();
		Vector trueVel = b->getVelocity();

		Vector rProj = trueVel.project(rNormal.perp2D());
		Vector lProj = trueVel.project(lNormal.perp2D());
		//ustawic sztuczna predkosc uprzdnio zapamietujac prawdziwa
		b->setVelocity(-lNormal*(2.0*sqrt(3)/3.0*rProj.magnitude())/*wspol zalezny od kata i szybkosci bili macierzystej*/);
		c = Contact((b->getPosition()-left->getPosition()).unit(),b,left);
		if(c.calculateRelativeVelocity()<0)
			c.resolve(duration);
		//podobnie ustawic odpowiednia predkosc
		b->setVelocity(-rNormal*(2.0*sqrt(3)/3.0*lProj.magnitude()));
		c = Contact((b->getPosition()-right->getPosition()).unit(),b,right);
		if(c.calculateRelativeVelocity()<0)
			c.resolve(duration);
		b->setVelocity(trueVel+lNormal*(2.0*sqrt(3)/3.0*rProj.magnitude())+rNormal*(2.0*sqrt(3)/3.0*lProj.magnitude()));
		//ustawic 0 i modlic sie ze nie zauwazy tej sciemy
		//		std::cout << b->getVelocity();

	}
	else if(left) {
//		std::cout << "Group resolving only left: " << left->getNumber() << ' ' << b->getNumber() << '\n';

		c = Contact((b->getPosition()-left->getPosition()).unit(),b,left);
		if(c.calculateRelativeVelocity()<0)
			c.resolve(duration);
		//		//		std::cout << b->getVelocity();

	}
	else if(right){
//		std::cout << "Group resolving only right: " << right->getNumber() << ' ' << b->getNumber() << '\n';

		Contact c((b->getPosition()-right->getPosition()).unit(),b,right);
		if(c.calculateRelativeVelocity()<0)
			c.resolve(duration);


	}

}
