/*
 * Contact.h
 *
 *  Created on: 2010-11-11
 *      Author: krejziwan
 */

#ifndef CONTACT_H_
#define CONTACT_H_

#include "RigidBody.h"
#include "../Common/Constants.h"

class Contact {
	RigidBody* body[2];
	Vector normal;
	real penetration;
	real restitution, cushionRestitution;

	void resolveVelocity(real duration);
	void resolveInterpenetration(real duration);
	void resolveGroupVelocity(real duration, Vector direction, RigidBody * body);
public:
	Contact(Vector _normal, RigidBody * b1, RigidBody * b2 = NULL): normal(_normal),penetration(0.0),restitution(RESTITUTION), cushionRestitution(CUSHION_RESTITUTION) {
		body[0] = b1;
		body[1] = b2;
	}
	Contact(Vector _normal, real p, RigidBody * b1, RigidBody * b2 = NULL): normal(_normal), penetration(p),
			restitution(RESTITUTION), cushionRestitution(CUSHION_RESTITUTION)  {
		body[0] = b1;
		body[1] = b2;
	}

	real calculateRelativeVelocity() const {
		Vector relVel = body[0]->getVelocity();
		if(body[1]) relVel -= body[1]->getVelocity();
		return relVel*normal;

	}

	void resolve(real duration) {
		resolveInterpenetration(duration);
		resolveVelocity(duration);
		if(body[1] && body[1]->isPartOfGroup())
			resolveGroupVelocity(duration, normal, body[1]);
	}


};

#endif /* CONTACT_H_ */
