/*
 * World.h
 *
 *  Created on: 2010-11-12
 *      Author: krejziwan
 */

#ifndef WORLD_H_
#define WORLD_H_


#include "../Common/Timer.h"
#include "RigidBody.h"
#include "Contact.h"
#include "ContactGenerator.h"
#include "ContactResolver.h"
#include "../Engine/INotifiable.h"
#include "../Engine/GameState.h"
#include <vector>
#include <algorithm>
#include <functional>
#include <iostream>

using namespace std;

	struct RegisteredBody {
		RigidBody * body;
		RegisteredBody * next;
	};

	struct RegisteredContactGen {
		ContactGenerator * gen;
		RegisteredContactGen * next;
	};

	struct RegisteredContactRes {
		ContactResolver * res;
		RegisteredContactRes * next;
	};

class Physics: public INotifiable {

	ContactResolver * resolver;
	ContactGenerator * generator;
//	RegisteredBody * regBody;
	std::vector<RigidBody *> bodies;
	std::vector<Contact> contacts;
	std::vector<Vector> pockets;
	real pocketSize;
	RigidBody * white;
	GameState *state;

	real power;

	void findBallsInPocket();
	bool isInPocket(const Vector& pos) {
		for(std::vector<Vector>::iterator it = pockets.begin(); it!=pockets.end();it++) {
			if((pos-(*it)).magnitude()<pocketSize) {
				return true;
			}
		}
		return false;
	}
public:
	Physics() {
		state=GameState::getInstance(); power = 1.0;
		pocketSize = POCKETSIZE;
		pockets.push_back(Vector(TABLEWIDTH/2.0,0.0,TABLELENGTH/2.0));
		pockets.push_back(Vector(TABLEWIDTH/2.0,0.0,-TABLELENGTH/2.0));
		pockets.push_back(Vector(-TABLEWIDTH/2.0,0.0,TABLELENGTH/2.0));
		pockets.push_back(Vector(-TABLEWIDTH/2.0,0.0,-TABLELENGTH/2.0));
		pockets.push_back(Vector(TABLEWIDTH/2.0,0.0, 0.0));
		pockets.push_back(Vector(-TABLEWIDTH/2.0,0.0, 0.0));
	};
	void startFrame();
	void integrate(real duration);
	unsigned generateContacts();
	void physicsUpdate(real durationInMicroSec);

	void add(RigidBody * b);

	void hitCueBall(const Vector& normal, real _power, const Vector& stick);
	RigidBody * getCueBall();
	void setCueBall(RigidBody * b) { white = b ;}
	void debug() {
		for(std::vector<RigidBody*>::iterator it = bodies.begin();it!=bodies.end();it++)
			std::cout << (*it)->getPosition();
		//for_each(bodies.begin(),bodies.end(),RigidBody::*getPosition);
	}
	void increasePower() {
		power*=1.1;
	}
	void decreasePower() {
		power *= 0.9;
	}

	void processNotification(ISubject *subject);
	void reset() {
		bodies.clear();
		state=GameState::getInstance(); power = 1.0;
		pocketSize = POCKETSIZE;
		pockets.push_back(Vector(TABLEWIDTH/2.0,0.0,TABLELENGTH/2.0));
		pockets.push_back(Vector(TABLEWIDTH/2.0,0.0,-TABLELENGTH/2.0));
		pockets.push_back(Vector(-TABLEWIDTH/2.0,0.0,TABLELENGTH/2.0));
		pockets.push_back(Vector(-TABLEWIDTH/2.0,0.0,-TABLELENGTH/2.0));
		pockets.push_back(Vector(TABLEWIDTH/2.0,0.0, 0.0));
		pockets.push_back(Vector(-TABLEWIDTH/2.0,0.0, 0.0));
	}

	bool isAnimationFinished() {
		for(std::vector<RigidBody *>::iterator it = bodies.begin(); it!= bodies.end(); ++it) {
			if((*it)->isAwake()) {
//				std::cout << (*it)->getNumber();
				return false;
			}
		}
		return true;
	}

};


#endif /* WORLD_H_ */
