/*
 * EventHandler.h
 *
 *  Created on: 2010-11-21
 *      Author: krejziwan
 */


#ifndef EVENTHANDLER_H_
#define EVENTHANDLER_H_


#include "../Engine/Camera.h"
#include "../Physics/Physics.h"
#include "../Engine/GameState.h"
#include <map>
#include <cstdlib>

class EventHandler:public INotifiable
{

	typedef void (EventHandler::*func)();
	static const int KEYBINDINGS=4;

	std::map<unsigned char, func>* mainMenuBindings()
	{
		std::map<unsigned char, func>* keybindings = new std::map<unsigned char, func>();
		keybindings->insert(std::pair<unsigned char, func>('y',&EventHandler::newGame));
		return keybindings;

	};
	std::map<unsigned char, func>* pauseBindings()
	{
		std::map<unsigned char, func>* keybindings = new std::map<unsigned char, func>();
		keybindings->insert(std::pair<unsigned char, func>('w',&EventHandler::slideforward));
				keybindings->insert(std::pair<unsigned char, func>('s',&EventHandler::slidebackward));
				keybindings->insert(std::pair<unsigned char, func>('a',&EventHandler::slideleft));
				keybindings->insert(std::pair<unsigned char, func>('d',&EventHandler::slideright));
				keybindings->insert(std::pair<unsigned char, func>('+',&EventHandler::slideup));
				keybindings->insert(std::pair<unsigned char, func>('-',&EventHandler::slidedown));
				keybindings->insert(std::pair<unsigned char, func>('Q',&EventHandler::rollleft));
				keybindings->insert(std::pair<unsigned char, func>('E',&EventHandler::rollright));
				keybindings->insert(std::pair<unsigned char, func>('A',&EventHandler::yawleft));
				keybindings->insert(std::pair<unsigned char, func>('D',&EventHandler::yawright));
				keybindings->insert(std::pair<unsigned char, func>('W',&EventHandler::pitchup));
				keybindings->insert(std::pair<unsigned char, func>('S',&EventHandler::pitchdown));
		keybindings->insert(std::pair<unsigned char, func>('p',&EventHandler::pause));
		keybindings->insert(std::pair<unsigned char, func>('\x1B',&EventHandler::exit_));
		return keybindings;
	};

	std::map<unsigned char, func>* gameBindings()
	{
		std::map<unsigned char, func>* keybindings = new std::map<unsigned char, func>();
		keybindings->insert(std::pair<unsigned char, func>('w',&EventHandler::slideforward));
			keybindings->insert(std::pair<unsigned char, func>('s',&EventHandler::slidebackward));
			keybindings->insert(std::pair<unsigned char, func>('a',&EventHandler::slideleft));
			keybindings->insert(std::pair<unsigned char, func>('d',&EventHandler::slideright));
			keybindings->insert(std::pair<unsigned char, func>('+',&EventHandler::slideup));
			keybindings->insert(std::pair<unsigned char, func>('-',&EventHandler::slidedown));
			keybindings->insert(std::pair<unsigned char, func>('Q',&EventHandler::rollleft));
			keybindings->insert(std::pair<unsigned char, func>('E',&EventHandler::rollright));
			keybindings->insert(std::pair<unsigned char, func>('A',&EventHandler::yawleft));
			keybindings->insert(std::pair<unsigned char, func>('D',&EventHandler::yawright));
			keybindings->insert(std::pair<unsigned char, func>('W',&EventHandler::pitchup));
			keybindings->insert(std::pair<unsigned char, func>('S',&EventHandler::pitchdown));
			keybindings->insert(std::pair<unsigned char, func>('h',&EventHandler::hitball));
			keybindings->insert(std::pair<unsigned char, func>('y',&EventHandler::debug));
			keybindings->insert(std::pair<unsigned char, func>('p',&EventHandler::pause));
			keybindings->insert(std::pair<unsigned char, func>('<',&EventHandler::decpower));
			keybindings->insert(std::pair<unsigned char, func>('>',&EventHandler::incpower));


		return keybindings;
	};

	std::map<unsigned char, func>* animationBindings()
		{
			std::map<unsigned char, func>* keybindings = new std::map<unsigned char, func>();
			keybindings->insert(std::pair<unsigned char, func>('w',&EventHandler::slideforward));
				keybindings->insert(std::pair<unsigned char, func>('s',&EventHandler::slidebackward));
				keybindings->insert(std::pair<unsigned char, func>('a',&EventHandler::slideleft));
				keybindings->insert(std::pair<unsigned char, func>('d',&EventHandler::slideright));
				keybindings->insert(std::pair<unsigned char, func>('+',&EventHandler::slideup));
				keybindings->insert(std::pair<unsigned char, func>('-',&EventHandler::slidedown));
				keybindings->insert(std::pair<unsigned char, func>('Q',&EventHandler::rollleft));
				keybindings->insert(std::pair<unsigned char, func>('E',&EventHandler::rollright));
				keybindings->insert(std::pair<unsigned char, func>('A',&EventHandler::yawleft));
				keybindings->insert(std::pair<unsigned char, func>('D',&EventHandler::yawright));
				keybindings->insert(std::pair<unsigned char, func>('W',&EventHandler::pitchup));
				keybindings->insert(std::pair<unsigned char, func>('S',&EventHandler::pitchdown));
				keybindings->insert(std::pair<unsigned char, func>('p',&EventHandler::pause));

			return keybindings;
		};


	Camera * cam;
	Physics * physics;
	GameState *state;

	real slidespeed;
	real rotatespeed;

	void slideforward() {cam->slide(0.0,0.0,-slidespeed);}
	void slidebackward() {cam->slide(0.0,0.0,slidespeed);}
	void slideleft() {cam->slide(-slidespeed,0.0,0.0);}
	void slideright() {cam->slide(slidespeed,0.0,0.0);}
	void slideup() {cam->slide(0.0,slidespeed,0.0);}
	void slidedown() {cam->slide(0.0,-slidespeed,0.0);}
	void rollleft() { cam->roll(-rotatespeed);}
	void rollright() {cam->roll(rotatespeed);}
	void yawleft() {cam ->rotate(rotatespeed);}
	void yawright() { cam->rotate(-rotatespeed);}
	void pitchup() { cam->pitch(rotatespeed);}
	void pitchdown() { cam->pitch(-rotatespeed);}
	void hitball() {
		state->setCurrentState(GameState::ANIMATION);
		Vector direction = cam->getN();
		physics->hitCueBall(direction,1,Vector(0.0,0.0,0.0));

	}
	void incpower() {physics->increasePower();}
	void decpower() {physics->decreasePower();}
	/**/
	void debug() {	physics->debug();}
	void newGame() { state->setCurrentState(GameState::NEW_GAME); }
	void mainMenu() { state->setCurrentState(GameState::MAIN_MENU); }
	void exit_() { exit(1);}
	void pause()
	{ switch(state->getCurrentState())
		{
		case GameState::HIT_BALL: state->setCurrentState(GameState::PAUSE_HIT);break;
		case GameState::PAUSE_HIT: state->setCurrentState(GameState::HIT_BALL); break;
		case GameState::PAUSE_ANI: state->setCurrentState(GameState::ANIMATION); break;
		case GameState::ANIMATION: state->setCurrentState(GameState::PAUSE_ANI); break;
		default:
			break;
		}
	}



	std::map<unsigned char, func>* activeKeyBindings;
	std::map<std::string,std::map<unsigned char, func>*> keyBindings;

public:
	EventHandler(Physics *physics=NULL,Camera *cam=NULL, GameState *state=NULL);
	virtual ~EventHandler();
	void processNotification(ISubject *subject);
	void handleKeyboardEvent(unsigned char key, int x, int y);
	void handleMouseEvent(int button, int state, int x, int y);

};

#endif /* EVENTHANDLER_H_ */
